@@ -17,18 +17,16 @@ sub _CB_TIMER {
     return 1;
 }
 
-sub time_out {
-    my ($self, $multi) = @_;
+sub _finish_handle {
+    my ($self, @args) = @_;
 
-    my $is_active = $self->SUPER::time_out($multi);
+    $self->SUPER::_finish_handle(@args);
 
-    # Sometimes (maybe depending on the curl version?) $is_active
-    # is 0 despite the presence of in-progress requests.
-    $is_active ||= %{ $self->{'callbacks'} } || %{ $self->{'deferred'} };
+    my $is_active = %{ $self->{'callbacks'} } || %{ $self->{'deferred'} };
 
     $self->CLEAR_TIMER() if !$is_active;
 
-    return $is_active;
+    return;
 }
 
 1;
